/***
*		Chop.c -- new & improved subdivider tool
*		 1996 Another Option   All rights reserved.
***/

/*
=============================================================================
|   DATE   | WHAT
=============================================================================
 28 Jan 96 | created
 30 Jan 96 | added stuff THAT WOULD ENHANCE DELAY.C like routeevent()
 13 Nov 96 | added process list and sysex support
*/

//#define FILEDUMP

#include <exec/types.h>
#include <exec/memory.h>
#include <dos/dos.h>
#include <intuition/intuition.h>
#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#include <clib/alib_protos.h>
#include <clib/alib_stdio_protos.h>
#include <string.h>
//#include <stdio.h>

#include "bars.h"
#include "edit.h"
#include "BarsMacros.h"
#include "BPLib.h"
#include "PopUpPipes.h"
#include "proto/chop_protos.h"
#include "chopw.h"
#include "ToolID.h"
#include "debug.h"

static UBYTE const versiontag[] = "$VER: Chopper 40.2 (14.11.96)";

#include "/icons/ChopDown.h"
#include "/icons/ChopUp.h"
#include "snapshotgadget.h"

/* tool flags */
/* byte 1 */
#define DIV_RANDOM         1
#define DIV_FIXED          2
#define DIV_CC             4
#define DIV_TRIGGER        8
#define ART_RANDOM      0x10
#define ART_EACHRANDOM  0x20
#define ART_FIXED       0x40
/* byte 2 */
#define ART_CC         0x100
#define TOOL_ON        0x200
#define NOTES_ONLY     0x400
/* got a switch on; do the next note */
#define DIV_WAITING    0x800
#define USE_SYSEX     0x1000
#define FREE_SYSEX    0x2000

#define DEFAULT_FLAGS (DIV_RANDOM|ART_FIXED|TOOL_ON|NOTES_ONLY|USE_SYSEX|FREE_SYSEX)

#define DIV_MASK (DIV_RANDOM | DIV_FIXED | DIV_CC | DIV_TRIGGER)
#define ART_MASK (ART_RANDOM | ART_FIXED | ART_CC | ART_EACHRANDOM)

/* routeflags */
#define BRANCH_DIVIDE     1
#define BRANCH_SOURCE     2
#define DISCARD_SOURCE    4
#define BRANCH_DIVCC      8
#define DISCARD_DIVCC  0x10
#define BRANCH_ARTCC   0x20
#define DISCARD_ARTCC  0x40

#define DEFAULT_ROUTEFLAGS (DISCARD_SOURCE | DISCARD_DIVCC | DISCARD_ARTCC)

#define MAX_IGNORE 1536
#define MAX_DIVISIONS 30

#define SX_LENGTH 13

struct ChopTool {
	struct Tool tool;
	UWORD flags;
	UWORD ignore; // ignore notes with durations less than this
	UBYTE routeflags;
	UBYTE div_cc;
	UBYTE div_max;
	UBYTE div_min;
	UBYTE art_cc;
	UBYTE art_max;
	UBYTE art_min;
	UBYTE divisions;    // stashes stuff from cc changes
	UBYTE articulation; // ditto
	UBYTE _byte_pad_;
	UWORD _word_pad_;
};

#define TOOL_SIZE sizeof(struct ChopTool) /* for ToolMaster */

extern struct IntuitionBase *IntuitionBase;
extern struct GfxBase *GfxBase;
extern struct LayersBase *LayersBase;
extern struct DOSBase *DOSBase;
extern struct ExecBase *SysBase;
extern struct Functions *functions;

/*==========================================================================
* inittool
*
*  INPUT: pointer to tool
* RETURN: none
*
* EFFECT: sets tool defaults
*=========================================================================*/

static void __regargs inittool(struct ChopTool *tool)

{
	tool->flags = DEFAULT_FLAGS;
	tool->routeflags = DEFAULT_ROUTEFLAGS;
	tool->div_cc = 63;
	tool->div_max = tool->divisions = 5;
	tool->div_min = 2;
	tool->art_cc = 62;
	tool->art_max = tool->articulation = 80;
	tool->art_min = 40;
	tool->ignore = 48;
	tool->tool.touched |= TOUCH_INIT;
}

/**************************************************************************/

static struct NoteEvent * __regargs dochoplist(struct ChopTool *tool,
                                               struct NoteEvent *event, ULONG div,
															  ULONG art)

{
	struct NoteEvent *chop;
	struct NoteEvent *prev = NULL;
	struct NoteEvent *list = NULL;
	long time_inc;
	long time;

//	if (div) {
#pragma msg 100 ignore push
		time_inc = (long)IDivU(event->duration,div);
#pragma msg 100 ignore pop
		if (time_inc < 1) time_inc = 1;
//	}
//	else time_inc = 1;
	time = event->time;
	while (div) {
		chop = (struct NoteEvent *)functions->dupeevent((struct Event *)event);
		if (chop) {
			if (!list) list = chop;
			if (prev) prev->next = chop;
			else prev = chop;
/*!! if duration < 1 this could be used on EACHRANDOM to spastically drop notes */
#pragma msg 100 ignore push
			chop->duration = IDivU(IMulU(time_inc,art),100);
#pragma msg 100 ignore pop
			if ((short)chop->duration < 1) chop->duration = 1;
			chop->time = time;
			if (tool->flags & ART_EACHRANDOM) {
				art = functions->random(tool->art_max-tool->art_min)+tool->art_min;
			}
			if (tool->routeflags & BRANCH_DIVIDE) {
				chop->tool = tool->tool.branch;
				chop->type |= EVENT_BRANCH;
			}
			else chop->tool = tool->tool.next;
			if (IsRealTime(chop) && IsNoteOn(chop)) {
				chop->next = CloneNoteOff(chop);
				prev = chop->next;
			}
			else {
				chop->next = NULL;
			}
			div--;
			time += time_inc;
		}
		else div = 0;
	}
	return(list);	
}

/**************************************************************************/
// this could be handy in bplib & in delay.c

static struct Event * __regargs routeevent(struct Event *event, struct Tool *tool,
                                           ULONG pass, ULONG branch) 
{
	if (pass) {
		event->tool = tool->next;
	}
	else if (branch) {
		event->tool = tool->branch;
		event->type |= EVENT_BRANCH;
	}
	else {
		functions->freeevent(event);
		event = NULL;
	}
	return(event);
}

/*******************/
// return the appropriate divisions value

static ULONG __regargs getdiv(struct ChopTool *tool)
{
	if (tool->flags & DIV_RANDOM) {
		return((ULONG)functions->random(tool->div_max-tool->div_min)+tool->div_min);
	}
	else if (tool->flags & DIV_FIXED) return(tool->div_max);
	else if (tool->flags & DIV_CC) return(tool->divisions);
}

/*******************/
// return the appropriate articulation value

static ULONG __regargs getart(struct ChopTool *tool)
{
	if (tool->flags & ART_FIXED) return(tool->art_max);
	else if (tool->flags & ART_CC) return(tool->articulation);
	else if (tool->flags & ART_RANDOM) {
		return((ULONG)functions->random(tool->art_max - tool->art_min)+tool->art_min);
	}
	else return(1); // articulation of each note is random
}

#define SECRET_MESSAGE 666

/*==========================================================================
* dosysex
*
*  INPUT: pointer to StringEvent
* RETURN: pointer to Event
*
* EFFECT: gets new tool state from sysex message
*=========================================================================*/

static __inline struct Event *dosysex(struct ChopTool *tool,
                                      struct StringEvent *event)
{
	unsigned char *sxdata;
	struct Message *msg;
	UWORD sxflag;

	if (tool->flags & USE_SYSEX) {
		if (sxdata = checksysex(event,TOOL_SYSEX_ID,tool->tool.toolid)) {
			sxflag = tool->flags & (USE_SYSEX | FREE_SYSEX);
			memcpy((void *)&tool->flags,(void *)sxdata,SX_LENGTH);
			if (sxflag & FREE_SYSEX) tool->flags |= FREE_SYSEX;
			if (tool->tool.window) {
				/* window is open; send a secret message to refresh */
				if (msg=(struct Message *)functions->myalloc(sizeof(struct Message),0)) {
					msg->mn_Node.ln_Type = NT_MESSAGE;
					msg->mn_ReplyPort = NULL;
					msg->mn_Length = SECRET_MESSAGE;
					PutMsg(tool->tool.window->UserPort,msg);
				}
			}
			if (tool->flags & FREE_SYSEX) {
				functions->freestring(event->string);
				functions->freeevent((struct Event *)event);
				event=NULL;
			}
		}
	}
	return((struct Event *)event);
}

/************************
 * scale cc value to range defined by min..max
 */

static UBYTE __regargs scalecc(UBYTE val, UBYTE  min, UBYTE max)
{
#pragma msg 100 ignore push
	return((UBYTE)(IDivU(IMulU(IDivU(IMulU(max-min+1,100),127),val),100) + min));
#pragma msg 100 pop
}

/*==========================================================================
* processevent
*
*  INPUT: pointer to Event
* RETURN: pointer to Event
*
* EFFECT: processes events one at a time
*=========================================================================*/

static struct Event * __saveds __stdargs processevent(struct NoteEvent *event)

{
   struct ChopTool *tool = (struct ChopTool *)event->tool;
	struct NoteEvent *chop; // the output list
//	ULONG div;
//	ULONG art;

	if (!tool->tool.touched) inittool(tool);

	if (event->status == MIDI_NOTEOFF) {
		functions->freeevent((struct Event *)event);
		event = NULL;
//		return(NULL);
	}
	else if ( (!(tool->flags & TOOL_ON)) || 
	     (IsNoteOn(event) && (event->duration < tool->ignore)) ) {
		event = (struct NoteEvent *)
		        routeevent((struct Event *)event,&tool->tool,
		                   !(tool->flags & (BRANCH_SOURCE|DISCARD_SOURCE)),
								 (tool->flags & BRANCH_SOURCE) && tool->tool.branch);
//		return((struct Event *)event);
	}
	else if (event->status==MIDI_CCHANGE) { /* get new cc stuff */
		if ((tool->flags & (DIV_CC|DIV_TRIGGER)) && (event->value==tool->div_cc)) {
			if (tool->flags & DIV_CC) {
				tool->divisions = scalecc(event->velocity,tool->div_min,tool->div_max);
			}
			else if (tool->flags & DIV_TRIGGER) { // set trigger bit
				if (SwitchOn(event->velocity)) {
					tool->flags |= DIV_WAITING;
				}
			}
/*!! HEY ASSHOLE. you can't do this here: if the event is freed you are going to
  !! be sorry if this is also the articulation cc. right?
  !!*/
			event = (struct NoteEvent *)
			        routeevent((struct Event *)event,&tool->tool,
			                   !(tool->routeflags & (DISCARD_DIVCC | BRANCH_DIVCC)),
									 (tool->routeflags & BRANCH_DIVCC) && tool->tool.branch);
		}
		if ((tool->flags & ART_CC) && (event->value == tool->art_cc)) {
			tool->articulation = scalecc(event->velocity,tool->art_min,tool->art_max);
			event = (struct NoteEvent *)routeevent((struct Event *)event,&tool->tool,
			        !(tool->routeflags & (DISCARD_ARTCC|BRANCH_ARTCC)),
					  (tool->routeflags & BRANCH_ARTCC) && tool->tool.branch);
		}

		else event->tool = tool->tool.next;
	} // is cc

/*!! either figure out what do with other stuff or lose the notes only gadget !!*/
	else if (IsNoteOn(event) && (tool->flags & NOTES_ONLY)) {
		if ((tool->flags & DIV_TRIGGER) && !(tool->flags & DIV_WAITING)) {
			chop = NULL;
			goto CHEAP_EXIT;
		}
		else if (tool->flags & DIV_TRIGGER) {
			tool->flags &= ~DIV_WAITING;
		}
		if ( ((tool->routeflags & BRANCH_DIVIDE) && tool->tool.branch) || 
		     (!(tool->routeflags & BRANCH_DIVIDE)) ) {
			chop = dochoplist(tool,event,getdiv(tool),getart(tool));
CHEAP_EXIT:
			event = (struct NoteEvent *)
			         routeevent((struct Event *)event,&tool->tool,
			                    !(tool->routeflags & (DISCARD_SOURCE|BRANCH_SOURCE)),
									  (tool->routeflags & BRANCH_SOURCE) && tool->tool.branch);
			if (event && IsNoteOn(event)) {
				event->next = CloneNoteOff(event);
				if (!event->next) {
					functions->freeevent((struct Event *)event);
					event = NULL;
				}
			}
			if (event) {
				if (IsNoteOn(event)) event->next->next = chop;
				else event->next = chop;
			}
			else event = chop;
		}
	}
	else if (event->status & EVENT_SYSX) {
		event = (struct NoteEvent *)dosysex(tool,(struct StringEvent *)event);
	}
//!! this is inconsistent with what happens if the tool is off or a note ignored
//	else event->tool = tool->tool.next;
	else event = (struct NoteEvent *)routeevent((struct Event *)event,&tool->tool,
		                   !(tool->flags & (BRANCH_SOURCE|DISCARD_SOURCE)),
								 (tool->flags & BRANCH_SOURCE) && tool->tool.branch);

	return((struct Event *)event);
}

/*==========================================================================
* processlist
*
*  INPUT: pointers to tool and list to process, start and stop times
* RETURN: pointer to inverted list
*
* EFFECT: processes a list of events
*=========================================================================*/

static struct Event * __stdargs __saveds processlist(struct ChopTool *tool,
                                                     struct NoteEvent *list,
                                                     long start, long stop)
{
	struct NoteEvent *chop = NULL; // the output list
	struct NoteEvent *event;
	struct NoteEvent *next;

	if (!tool->tool.touched) inittool(tool);
	if (start < 0) start = 0;
	tool->flags &= ~DIV_WAITING; /* in case of an odd number of ccs last time  */
	event = list;
	for (; (start <= stop) && event ; start=(next)?next->time:stop+1, event=next) {
		next = event->next;
		if (IsNoteOn(event) && (event->duration < tool->ignore)) {
			if (!(tool->flags & (BRANCH_SOURCE|DISCARD_SOURCE))) {
				event->tool = tool->tool.next;
			}
			else if ((tool->flags & BRANCH_SOURCE) && tool->tool.branch) {
				event->tool = tool->tool.branch;
				event->type |= EVENT_BRANCH;
			}
			else event->time = -1;
		}

		else if (event->status==MIDI_CCHANGE) { /* get new cc stuff */
			if ( (tool->flags & (DIV_CC|DIV_TRIGGER)) && 
			     (event->value==tool->div_cc) ) {
				if (tool->flags & DIV_CC) {
					tool->divisions =
						           scalecc(event->velocity,tool->div_min,tool->div_max);
				}
				else if (tool->flags & DIV_TRIGGER) { // set trigger bit
					if (SwitchOn(event->velocity)) {
						tool->flags |= DIV_WAITING;
					}
				}
				if (!(tool->routeflags & (DISCARD_DIVCC | BRANCH_DIVCC))) {
					event->tool = tool->tool.next;
				}
				else if ((tool->routeflags & BRANCH_DIVCC) && tool->tool.branch) {
					event->tool = tool->tool.branch;
					event->type |= EVENT_BRANCH;
				}
				else event->time = -1;
			}
			if ((tool->flags & ART_CC) && (event->value == tool->art_cc)) {
				tool->articulation = scalecc(event->velocity,tool->art_min,tool->art_max);
				if (!(tool->routeflags & (DISCARD_ARTCC|BRANCH_ARTCC))) {
					/* if artcc==divcc it already got pointed to the next tool */
					if (event->tool==(struct Tool *)tool)
						event->tool = tool->tool.next;
				}
				else if ((tool->routeflags & BRANCH_ARTCC) && tool->tool.branch) {
					event->tool = tool->tool.branch;
					event->type |= EVENT_BRANCH;
				}
				else event->time = -1;
			}
			else event->tool = tool->tool.next;
		} // is cc
/*!! either figure out what do with other stuff or lose the notes only gadget !!*/
		else if (IsNoteOn(event) && (tool->flags & NOTES_ONLY)) {
			if ((tool->flags & DIV_TRIGGER) && !(tool->flags & DIV_WAITING)) {
				goto CHEAP_JUMP;
			}
			else if (tool->flags & DIV_TRIGGER) {
				tool->flags &= ~DIV_WAITING;
			}
			if ( ((tool->routeflags & BRANCH_DIVIDE) && tool->tool.branch) || 
			     (!(tool->routeflags & BRANCH_DIVIDE)) ) {
				chop = (struct NoteEvent *)functions->List_Cat((struct ListItem *)chop,
				  (struct ListItem *)dochoplist(tool,event,getdiv(tool),getart(tool)));
CHEAP_JUMP:
				if (!(tool->routeflags & (DISCARD_SOURCE|BRANCH_SOURCE))) {
					event->tool = tool->tool.next;
				}
				else if ((tool->routeflags & BRANCH_SOURCE) && tool->tool.branch) {
					event->tool = tool->tool.branch;
					event->type |= EVENT_BRANCH;
				}
				else event->time = -1;
			}
		}
//!! this is inconsistent with what happens if the tool is off or a note ignored
//	else event->tool = tool->tool.next;
		else {
			if (!(tool->flags & (DISCARD_SOURCE|BRANCH_SOURCE))) {
				event->tool = tool->tool.next;
			}
			else if ((tool->flags & BRANCH_SOURCE) && tool->tool.branch) {
				event->tool = tool->tool.branch;
				event->type |= EVENT_BRANCH;
			}
			else event->time = -1;
		}
	}
	
	list = (struct NoteEvent *)
	       functions->List_Cat((struct ListItem *)list,(struct ListItem *)chop);
	list = (struct NoteEvent *)functions->sorteventlist((struct Event *)list);
	for ( event=list ; event ; event=list ) {
		if (event->time >= 0) break;
		else {
			list = event->next;
			functions->freeevent((struct Event *)event);
		}
	}
	return((struct Event *)list);
}



/*==========================================================================
* dosnapshot
*
*  INPUT: pointer to tool
* RETURN: none
*
* EFFECT: makes a sysex message and posts it
*=========================================================================*/

static void dosnapshot(struct ChopTool *tool)

{
	UBYTE sx[SX_LENGTH];
	struct StringEvent *event;
	
	memcpy(sx,&tool->flags,SX_LENGTH); 
	if (event = BuildSnapshot(&tool->tool, 
	                          functions->running ? functions->timenow : -1,
	                          sx, SX_LENGTH)) {
		PostSnapshot(&tool->tool,event);
		if (tool->tool.track->edit->window) {
			functions->updateobject(&functions->inittempo,0); /* redisplay */
			/* how about sending a refresh message to the window? how? */
		}
	}
}


static struct ChopTool *selectedtool = NULL;

/***
 *** displays time m.bb.ccc, 0=='Nothing', '<' preceding time
 ***/

static long __saveds __stdargs hiloBeatMe(struct Window *w,struct Gadget *g,long  x)
{
	char text[13];

	if (x==0) {
		strcpy(text,"  Nothing  ");
	}
	else {
		functions->lengthtostring(selectedtool->tool.clip,0,x,text);
		text[2] = '<';
	}
	SetRPort(w->RPort, g->LeftEdge+g->Width+XOffset-4, g->TopEdge+YOffset,
            HILO_APEN,HILO_BPEN, HILO_DRAWMODE);
	Text(w->RPort,&text[2],9);
	return((long)x);
}

#define tw tool->tool.window
#define GDOWN 200
#define GUP   300

#define REFRESH_DIV 1
#define REFRESH_ART 2
#define REFRESH_POPPIPES 4
#define REFRESH_DIVPOTS  8
#define REFRESH_ARTPOTS 16
#define REFRESH_SOURCE  32

/*==========================================================================
* edittool
*
*  INPUT: pointer to tool
* RETURN: none
*
* EFFECT: ui handler
*=========================================================================*/

static void __saveds __stdargs edittool(struct ChopTool *tool)

{
	struct IntuiMessage *message;
	long class; 
	long code;
	struct Gadget *gadget;
	struct NewWindow *nw;
	struct Menu *menu;
	struct Menu *pipemenu;
	struct MenuItem *menuitem;
	short menunum;
	UBYTE refresh;

	if (!(tool->tool.touched & TOUCH_INIT)) inittool(tool);
	pipemenu = functions->DupeMenu(&PopPipeMenu);
	if (!pipemenu) return;
	menu = functions->DupeMenu(&MenuList1);
	if (!menu) return;
	if (tool->flags & USE_SYSEX) menu->FirstItem->Flags |= CHECKED;
	else menu->FirstItem->Flags &= ~CHECKED;
	if (tool->flags & FREE_SYSEX) menu->FirstItem->NextItem->Flags |= CHECKED;
	else menu->FirstItem->NextItem->Flags &= ~CHECKED;
	refresh = InToolPad((struct Tool *)tool);
	nw=OpenEmbossedWindowSlop(&NewWindowStructure1,&tool->tool,0,WTITLE,
	                          (refresh) ? 0 : 20);
	if (!nw) return;
	if (!refresh) AddSnapshotGadget(tw);
	SetMenuStrip(tw,menu);
	DrawBevelBox(tw->RPort,DIVBORDERX,DIVBORDERY,DIVBORDERX2,DIVBORDERY2,-1,-1,1);
	DrawBevelBox(tw->RPort,ARTBORDERX,ARTBORDERY,ARTBORDERX2,ARTBORDERY2,-1,-1,1);
	DrawBevelBox(tw->RPort,SRCBORDERX,SRCBORDERY,SRCBORDERX2,SRCBORDERY2,-1,-1,1);
	PrintIText(tw->RPort,&IntuiTextList1,0,0);
	EmbossOnGList(tw,&GadgetList1,GCOUNT,1);
	functions->EnableGadget(tw,ONG,!refresh); // not valid in toolpad
	functions->FatEmbossedPropOn(tw,DIV_MAX,DIV_MAX+GDOWN,DIV_MAX+GUP,hilo,
	                             MAX_DIVISIONS+1,1);
	functions->FatEmbossedPropOn(tw,DIV_MIN,DIV_MIN+GDOWN,DIV_MIN+GUP,hilo,
	                             MAX_DIVISIONS+1,1);
	functions->FatEmbossedPropOn(tw,ART_MAX,ART_MAX+GDOWN,ART_MAX+GUP,
	                             hiloPercent,101,1);
	functions->FatEmbossedPropOn(tw,ART_MIN,ART_MIN+GDOWN,ART_MIN+GUP,
	                             hiloPercent,101,1);
	functions->FatEmbossedPropOn(tw,DIV_CONTROLLER,DIV_CONTROLLER+GDOWN,DIV_CONTROLLER+GUP,
	                             hilo,121,1);
	functions->FatEmbossedPropOn(tw,ART_CONTROLLER,ART_CONTROLLER+GDOWN,ART_CONTROLLER+GUP,
	                             hilo,121,1);
	selectedtool = tool;
	functions->FatEmbossedPropOn(tw,IGNORE,IGNORE+GDOWN,IGNORE+GUP,hiloBeatMe,
	                             MAX_IGNORE+1,1);
	refresh = 255;
//	SetRefreshOK(&tool->tool);

	/*** ui handling ***/
	for ( ; ; ) {
		if (refresh) {
			if (refresh & REFRESH_SOURCE) {
				functions->SelectEmbossed(tw,NOTES_ONLYG,tool->flags & NOTES_ONLY);
				functions->SelectEmbossed(tw,ONG,tool->flags & TOOL_ON);
				functions->DrawEmbossed(tw,NOTES_ONLYG);
				functions->DrawEmbossed(tw,ONG);
				selectedtool = tool;
				functions->ModifyEmbossedProp(tw,IGNORE,tool->ignore,0,0,0,0,0);
				functions->DrawEmbossedProp(tw,IGNORE);
			}
			if (refresh & REFRESH_DIV) {
				functions->SelectEmbossed(tw,DIV_RANDOMG,tool->flags & DIV_RANDOM);
				functions->SelectEmbossed(tw,DIV_FIXEDG,tool->flags & DIV_FIXED);
				functions->SelectEmbossed(tw,DIV_CCG,tool->flags & DIV_CC);
				functions->SelectEmbossed(tw,DIV_TRIGGERG,tool->flags & DIV_TRIGGER);
				DrawEmbossedGList(tw,functions->GetGadget(tw,DIV_RANDOMG),4);
				functions->EnableGadget(tw,POP_DIVCC,tool->flags & (DIV_CC | DIV_TRIGGER));
				refresh |= REFRESH_DIVPOTS;
			}
			if (refresh & REFRESH_ART) {
				functions->SelectEmbossed(tw,ART_RANDOMG,tool->flags & ART_RANDOM);
				functions->SelectEmbossed(tw,ART_EACHRANDOMG,tool->flags & ART_EACHRANDOM);
				functions->SelectEmbossed(tw,ART_FIXEDG,tool->flags & ART_FIXED);
				functions->SelectEmbossed(tw,ART_CCG,tool->flags & ART_CC);
				DrawEmbossedGList(tw,functions->GetGadget(tw,ART_RANDOMG),4);
				functions->EnableGadget(tw,POP_ARTCC,tool->flags & ART_CC);
				refresh |= REFRESH_ARTPOTS;
			}
			if (refresh & REFRESH_DIVPOTS) {
				if (tool->div_min < 1) tool->div_min = 1;
				if ((!(tool->flags & DIV_FIXED)) && (tool->div_max < tool->div_min))
					tool->div_max = tool->div_min;
				else if (tool->div_max < 1) tool->div_max = 1;
				if (tool->flags & DIV_CC) tool->divisions = tool->div_min;
				functions->ModifyEmbossedProp(tw,DIV_MAX,tool->div_max,0,0,0,0,0);
				SetGadgetText(tw,DIV_MAX,(tool->flags & DIV_FIXED) ? FixTxt : MaxTxt);
				functions->DrawEmbossedProp(tw,DIV_MAX);
				functions->ModifyEmbossedProp(tw,DIV_MIN,tool->div_min,0,0,0,0,0);
				EnableFatProp(tw, DIV_MIN,DIV_MIN+GDOWN,DIV_MIN+GUP,
				              tool->flags & (DIV_CC | DIV_TRIGGER | DIV_RANDOM));
				functions->DrawEmbossedProp(tw,DIV_MIN);
				functions->ModifyEmbossedProp(tw,DIV_CONTROLLER,tool->div_cc,0,0,0,0,0);
				EnableFatProp(tw, DIV_CONTROLLER,DIV_CONTROLLER+GDOWN,DIV_CONTROLLER+GUP,
				              tool->flags & (DIV_CC | DIV_TRIGGER));
				functions->DrawEmbossedProp(tw,DIV_CONTROLLER);
			}
			if (refresh & REFRESH_ARTPOTS) {
				if (tool->art_min < 1) tool->art_min = 1;
				if ((!(tool->flags & ART_FIXED)) && (tool->art_max < tool->art_min))
					tool->art_max = tool->art_min;
				else if (tool->art_max < 1) tool->art_max = 1;
				if (tool->flags & ART_CC) tool->articulation = tool->art_min;
				functions->ModifyEmbossedProp(tw,ART_MAX,tool->art_max,0,0,0,0,0);
				SetGadgetText(tw,ART_MAX,(tool->flags & ART_FIXED) ? FixTxt : MaxTxt);
				functions->DrawEmbossedProp(tw,ART_MAX);
				functions->ModifyEmbossedProp(tw,ART_MIN,tool->art_min,0,0,0,0,0);
				EnableFatProp(tw, ART_MIN,ART_MIN+GDOWN,ART_MIN+GUP,
				              tool->flags & (ART_CC | ART_EACHRANDOM | ART_RANDOM));
				functions->DrawEmbossedProp(tw,ART_MIN);
				functions->ModifyEmbossedProp(tw,ART_CONTROLLER,tool->art_cc,0,0,0,0,0);
				EnableFatProp(tw, ART_CONTROLLER,ART_CONTROLLER+GDOWN,ART_CONTROLLER+GUP,
				              tool->flags & ART_CC);
				functions->DrawEmbossedProp(tw,ART_CONTROLLER);
			}

			if (refresh & REFRESH_POPPIPES) {
				gadget = functions->GetGadget(tw,POP_DIVCC);
				if (tool->routeflags & DISCARD_DIVCC)
					gadget->GadgetRender = (APTR)&PlugImage;
				else if (tool->routeflags & BRANCH_DIVCC)
					gadget->GadgetRender = (APTR)&ElbowImage;
				else gadget->GadgetRender = (APTR)&PipeImage;

				gadget = functions->GetGadget(tw,POP_ARTCC);
				if (tool->routeflags & DISCARD_ARTCC)
					gadget->GadgetRender = (APTR)&PlugImage;
				else if (tool->routeflags & BRANCH_ARTCC)
					gadget->GadgetRender = (APTR)&ElbowImage;
				else gadget->GadgetRender = (APTR)&PipeImage;

				gadget = functions->GetGadget(tw,POP_SOURCE);
				if (tool->routeflags & DISCARD_SOURCE) 
					gadget->GadgetRender = (APTR)&PlugImage;
				else if (tool->routeflags & BRANCH_SOURCE)
					gadget->GadgetRender = (APTR)&ElbowImage;
				else gadget->GadgetRender = (APTR)&PipeImage;

				gadget = functions->GetGadget(tw,POP_DIVISIONS);
				if (tool->routeflags & BRANCH_DIVIDE)
					gadget->GadgetRender = (APTR)&ElbowImage;
				else gadget->GadgetRender = (APTR)&PipeImage;

				RefreshGList(gadget,tw,NULL,4);
			}
			refresh=0;
		}
		message = (struct IntuiMessage *)functions->GetIntuiMessage(tw);
		if (message->ExecMessage.mn_Length == SECRET_MESSAGE) { // ooh, sysex!
			refresh = 0xFF;
			functions->myfree((char *)message,sizeof(struct Message));
			continue;
		}
		/* else */
		class = message->Class;
		code = message->Code;
		gadget = (struct Gadget *)message->IAddress;
		ReplyMsg((struct Message *)message);
		class = functions->SystemGadgets(tw,class,gadget,code);

		if (class == CLOSEWINDOW) {
			selectedtool = NULL;
			ResetRefreshOK(&tool->tool);
			break;
		}

		else if (class == MENUPICK) {
			while (code != MENUNULL) {
				menuitem = ItemAddress(menu,code);
				switch (ITEMNUM(code)) {
					case 0 :
						tool->flags ^= USE_SYSEX;
						break;
					case 1 :
						tool->flags ^= FREE_SYSEX;
						break;
					case 2 :
						dosnapshot(tool);
						break;
				}
				code = menuitem->NextSelect;
			}
		}

		else if (class == GADGETUP) {
			switch (gadget->GadgetID) {
				case DIV_RANDOMG :
					tool->flags ^= DIV_RANDOM;
					tool->flags &= ~DIV_MASK | (DIV_RANDOM | DIV_TRIGGER);
					if (!tool->flags & DIV_MASK) tool->flags |= DIV_RANDOM;
					refresh = REFRESH_DIV;
					break;
				case DIV_FIXEDG :
					tool->flags = (tool->flags & ~DIV_MASK) | DIV_FIXED;
					refresh = REFRESH_DIV;
					break;
				case DIV_CCG :
					tool->flags = (tool->flags & ~DIV_MASK) | DIV_CC;
					refresh = REFRESH_DIV;
					break;
				case DIV_TRIGGERG :
					tool->flags = (tool->flags & (~DIV_MASK | DIV_RANDOM)) | DIV_TRIGGER;
					refresh = REFRESH_DIV;
					break;
				case ART_RANDOMG :
					tool->flags = (tool->flags & ~ART_MASK) | ART_RANDOM;
					refresh = REFRESH_ART;
					break;
				case ART_EACHRANDOMG :
					tool->flags = (tool->flags & ~ART_MASK) | ART_EACHRANDOM;
					refresh = REFRESH_ART;
					break;
				case ART_CCG :
					tool->flags = (tool->flags & ~ART_MASK) | ART_CC;
					refresh = REFRESH_ART;
					break;
				case ART_FIXEDG :
					tool->flags = (tool->flags & ~ART_MASK) | ART_FIXED;
					refresh = REFRESH_ART;
					break;
				case NOTES_ONLYG :
					tool->flags ^= NOTES_ONLY;
					refresh = REFRESH_SOURCE;
					break;
				case ONG :
					tool->flags ^= TOOL_ON;
					refresh = REFRESH_SOURCE;
					break;
				case SNAPSHOT_GADGET_ID :
					dosnapshot(tool);
					break;
			}
		}
		else if (class == GADGETDOWN) {
			switch (gadget->GadgetID) {
				case POP_SOURCE :
					menunum = (short)functions->PopUpMenu(tw,pipemenu);
					if (ITEMNUM(menunum) < 3) {
						switch (ITEMNUM(menunum)) {
							case PIPE_ITEM : // pipe selected
								tool->routeflags &= ~(DISCARD_SOURCE|BRANCH_SOURCE);
								break;
							case PLUG_ITEM : // plug selected
								tool->routeflags = (tool->routeflags & ~BRANCH_SOURCE)|DISCARD_SOURCE;
								break;
							case ELBOW_ITEM : // elbow selected
								tool->routeflags = (tool->routeflags & ~DISCARD_SOURCE)|BRANCH_SOURCE;
								break;
						}
						refresh = REFRESH_POPPIPES;
					}
					break;
				case POP_DIVISIONS :
					functions->EnableMenuItem(pipemenu,0,PLUG_ITEM,0,0); // no plug
					menunum = (short)functions->PopUpMenu(tw,pipemenu);
					if (ITEMNUM(menunum) < 3) {
						switch (ITEMNUM(menunum)) {
							case PIPE_ITEM : // pipe selected
								tool->routeflags &= ~BRANCH_DIVIDE;
								break;
							case ELBOW_ITEM : // elbow selected
								tool->routeflags |= BRANCH_DIVIDE;
								break;
						}
						refresh = REFRESH_POPPIPES;
					}
					functions->EnableMenuItem(pipemenu,0,1,0,1);
					break;
				case POP_ARTCC :
					menunum = (short)functions->PopUpMenu(tw,pipemenu);
					if (ITEMNUM(menunum) < 3) {
						switch (ITEMNUM(menunum)) {
							case PIPE_ITEM : // pipe selected
								tool->routeflags &= ~(DISCARD_ARTCC | BRANCH_ARTCC);
								break;
							case PLUG_ITEM : // plug selected
								tool->routeflags = (tool->routeflags & ~BRANCH_ARTCC)|DISCARD_ARTCC;
								break;
							case ELBOW_ITEM :
								tool->routeflags = (tool->routeflags & ~DISCARD_ARTCC)|BRANCH_ARTCC;
								break;
						}
						refresh = REFRESH_POPPIPES;
					}
					break;
				case POP_DIVCC :
					menunum = (short)functions->PopUpMenu(tw,pipemenu);
					if (ITEMNUM(menunum) < 3) {
						switch (ITEMNUM(menunum)) {
							case PIPE_ITEM : // pipe selected
								tool->routeflags &= ~(DISCARD_DIVCC | BRANCH_DIVCC);
								break;
							case PLUG_ITEM : // plug selected
								tool->routeflags = (tool->routeflags & ~BRANCH_DIVCC)|DISCARD_DIVCC;
								break;
							case ELBOW_ITEM :
								tool->routeflags = (tool->routeflags & ~DISCARD_DIVCC)|BRANCH_DIVCC;
								break;
						}
						refresh = REFRESH_POPPIPES;
					}
					break;

				case DIV_MIN :
					tool->div_min = functions->DragEmbossedProp(tw,DIV_MIN);
					refresh= REFRESH_DIVPOTS;
					break;
				case DIV_MIN+GUP :
					tool->div_min = functions->ShiftEmbossedProp(tw,DIV_MIN,1,0);
					refresh= REFRESH_DIVPOTS;
					break;
				case DIV_MIN+GDOWN :
					tool->div_min = functions->ShiftEmbossedProp(tw,DIV_MIN,-1,0);
					refresh= REFRESH_DIVPOTS;
					break;
				case DIV_MAX :
					tool->div_max = functions->DragEmbossedProp(tw,DIV_MAX);
					refresh= REFRESH_DIVPOTS;
					break;
				case DIV_MAX+GUP :
					tool->div_max = functions->ShiftEmbossedProp(tw,DIV_MAX,1,0);
					refresh= REFRESH_DIVPOTS;
					break;
				case DIV_MAX+GDOWN :
					tool->div_max = functions->ShiftEmbossedProp(tw,DIV_MAX,-1,0);
					refresh= REFRESH_DIVPOTS;
					break;
				case DIV_CONTROLLER :
					tool->div_cc = functions->DragEmbossedProp(tw,DIV_CONTROLLER);
					refresh= REFRESH_DIVPOTS;
					break;
				case DIV_CONTROLLER+GUP :
					tool->div_cc = functions->ShiftEmbossedProp(tw,DIV_CONTROLLER,1,0);
					refresh= REFRESH_DIVPOTS;
					break;
				case DIV_CONTROLLER+GDOWN :
					tool->div_cc = functions->ShiftEmbossedProp(tw,DIV_CONTROLLER,-1,0);
					refresh= REFRESH_DIVPOTS;
					break;

				case ART_MIN :
					tool->art_min = functions->DragEmbossedProp(tw,ART_MIN);
					refresh= REFRESH_ARTPOTS;
					break;
				case ART_MIN+GUP :
					tool->art_min = functions->ShiftEmbossedProp(tw,ART_MIN,1,0);
					refresh= REFRESH_ARTPOTS;
					break;
				case ART_MIN+GDOWN :
					tool->art_min = functions->ShiftEmbossedProp(tw,ART_MIN,-1,0);
					refresh= REFRESH_ARTPOTS;
					break;
				case ART_MAX :
					tool->art_max = functions->DragEmbossedProp(tw,ART_MAX);
					refresh= REFRESH_ARTPOTS;
					break;
				case ART_MAX+GUP :
					tool->art_max = functions->ShiftEmbossedProp(tw,ART_MAX,1,0);
					refresh= REFRESH_ARTPOTS;
					break;
				case ART_MAX+GDOWN :
					tool->art_max = functions->ShiftEmbossedProp(tw,ART_MAX,-1,0);
					refresh= REFRESH_ARTPOTS;
					break;
				case ART_CONTROLLER :
					tool->art_cc = functions->DragEmbossedProp(tw,ART_CONTROLLER);
					refresh= REFRESH_ARTPOTS;
					break;
				case ART_CONTROLLER+GUP :
					tool->art_cc = functions->ShiftEmbossedProp(tw,ART_CONTROLLER,1,0);
					refresh= REFRESH_ARTPOTS;
					break;
				case ART_CONTROLLER+GDOWN :
					tool->art_cc = functions->ShiftEmbossedProp(tw,ART_CONTROLLER,-1,0);
					refresh= REFRESH_ARTPOTS;
					break;

				case IGNORE :
					tool->ignore = functions->DragEmbossedProp(tw,IGNORE);
					refresh= REFRESH_DIVPOTS;
					break;
				case IGNORE+GUP :
					tool->ignore = functions->ShiftEmbossedProp(tw,IGNORE,1,0);
					refresh= REFRESH_DIVPOTS;
					break;
				case IGNORE+GDOWN :
					tool->ignore = functions->ShiftEmbossedProp(tw,IGNORE,-1,0);
					refresh= REFRESH_DIVPOTS;
					break;
			}
		}
	}

	EmbossOffGList(tw,&GadgetList1,GCOUNT);
	functions->FatEmbossedPropOff(tw,DIV_MAX,DIV_MAX+GDOWN,DIV_MAX+GUP);
	functions->FatEmbossedPropOff(tw,DIV_MIN,DIV_MIN+GDOWN,DIV_MIN+GUP);
	functions->FatEmbossedPropOff(tw,DIV_CONTROLLER,DIV_CONTROLLER+GDOWN,DIV_CONTROLLER+GUP);
	functions->FatEmbossedPropOff(tw,ART_MAX,ART_MAX+GDOWN,ART_MAX+GUP);
	functions->FatEmbossedPropOff(tw,ART_MIN,ART_MIN+GDOWN,ART_MIN+GUP);
	functions->FatEmbossedPropOff(tw,ART_CONTROLLER,ART_CONTROLLER+GDOWN,ART_CONTROLLER+GUP);
	functions->FatEmbossedPropOff(tw,IGNORE,IGNORE+GDOWN,IGNORE+GUP);
	functions->DeleteMenu(pipemenu);
	functions->DeleteMenu(menu);
	NewCloseEmbossedWindow(&tool->tool,nw);
}

/*****
/*==========================================================================
* loadtool
*
*  INPUT: file id, size of chunk to read
* RETURN: pointer to allocated tool, or NULL
*
* EFFECT: allocates and initializes a tool with data read from disk
*=========================================================================*/

static struct Tool * __saveds __stdargs loadtool(long file, long size)

{
	struct ChopTool *tool;

	if (tool = (struct ChopTool *)functions->myalloc(TOOL_SIZE,0)) {
		functions->fastread(file,(char *)tool,TOOL_SIZE);
		tool->tool.touched &= ~TOUCH_REFRESHOK;
		size -= TOOL_SIZE;
	}
	if (size) { /* leftovers */
		functions->fastseek(file,size,0);
	}
	return((struct Tool *)tool);
}
****/

static struct ToolMaster master;

/*==========================================================================
* inittoolmaster
*
*  INPUT: none
* RETURN: pointer to ToolMaster
*
* EFFECT: initializes.
*=========================================================================*/

struct ToolMaster * __saveds __stdargs inittoolmaster(void)

{
	memset((char *)&master,0,sizeof(struct ToolMaster));
	master.toolsize = TOOL_SIZE;
	master.toolid = Chpr_TOOLID;
	master.edittool = edittool;
	master.processevent = processevent;
	master.processlist = processlist;

/*** these are the optional routines
	master.loadtool = loadtool;
	master.savesize = savesize;
	master.savetool = savetool;
	master.createtool = createtool;
	master.deletetool = deletetool;
	master.drawtoolicon = drawicon;
***/
	master.image = &IconDown;
	master.upimage = &IconUp;
	strcpy(master.name,WTITLE);
	master.tooltype = TOOL_NORMAL | TOOL_BRANCHOUT;


	return(&master);
}
